__version__ = "1.1.4"

from .apc_bias import apc_bias
from .bias import bias
from .brier_score import brier_score
from .centered_rms_dev import centered_rms_dev
from .check_duplicate_stats import check_duplicate_stats
from .check_on_off import check_on_off
from .check_taylor_stats import check_taylor_stats
from .get_target_diagram_axes import get_target_diagram_axes
from .get_target_diagram_options import get_target_diagram_options
from .get_taylor_diagram_axes import get_taylor_diagram_axes
from .get_taylor_diagram_options import get_taylor_diagram_options
from .index_agreement import index_agreement
from .kge09 import kge09

kge = kge09
from .kge12 import kge12
from .nse import nse
from .overlay_target_diagram_circles import overlay_target_diagram_circles
from .overlay_taylor_diagram_circles import overlay_taylor_diagram_circles
from .overlay_taylor_diagram_lines import overlay_taylor_diagram_lines
from .pc_bias import pc_bias
from .plot_pattern_diagram_colorbar import plot_pattern_diagram_colorbar
from .plot_pattern_diagram_markers import plot_pattern_diagram_markers
from .plot_target_axes import plot_target_axes
from .plot_taylor_axes import plot_taylor_axes
from .plot_taylor_obs import plot_taylor_obs
from .report_duplicate_stats import report_duplicate_stats
from .rmsd import rmsd
from .skill_score_brier import skill_score_brier
from .skill_score_murphy import skill_score_murphy
from .target_diagram import target_diagram
from .target_statistics import target_statistics
from .taylor_diagram import taylor_diagram
from .taylor_statistics import taylor_statistics
from .write_stats import write_stats
from .write_target_stats import write_target_stats
from .write_taylor_stats import write_taylor_stats
